"""Invoke as `ynab-import' or `python -m ynab_import'.
"""
import typer

from ynab_import.setup import create_reader, create_writer
from ynab_import.common.settings import ReaderKind, WriterKind
from ynab_import.core import import_transactions

app = typer.Typer(add_completion=False)


@app.command()
def cli(
    reader: ReaderKind = ReaderKind.swedbank_csv,
    writer: WriterKind = WriterKind.ynab_api,
):
    reader = create_reader(reader)
    writer = create_writer(writer)
    count = import_transactions(read=reader, write=writer)
    message = f"Transaction(s) imported: N {count} " + typer.style(
        "good", fg=typer.colors.GREEN, bold=True
    )
    typer.echo(message)


app()



from enum import Enum

from pydantic import BaseSettings


class ReaderKind(str, Enum):
    swedbank_csv = "swedbank_csv"


class WriterKind(str, Enum):
    ynab_api = "ynab_api"
    memory = "memory"
    stdout = "stdout"


class Settings(BaseSettings):
    reader: ReaderKind
    writer: WriterKind
    read_from_path: str = ""

    ynab_api_key: str
    ynab_budget_id: str
    ynab_account_id: str

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"


settings = Settings()



from ynab_import.common.settings import ReaderKind, WriterKind, settings
from ynab_import.core.interactions import IReadRepository, IWriteRepository
from ynab_import.infra.http import RequestsClient
from ynab_import.infra.ynab import YnabAPIBasedRepository
from ynab_import.swedbank.providers import SwedbankCSVReader


def create_reader(reader: ReaderKind) -> IReadRepository:
    if reader == ReaderKind.swedbank_csv:
        return SwedbankCSVReader(source_file_path=settings.read_from_path)

    raise NotImplementedError(f"{reader} reader is not implemented.")


def create_writer(writer: WriterKind) -> IWriteRepository:
    if writer == WriterKind.ynab_api:
        return _create_ynab_api_reader()
    raise NotImplementedError(f"{writer} writer is not implemented.")


def _create_ynab_api_reader():
    http_client = (
        RequestsClient()
        .with_base_url("https://api.youneedabudget.com/v1/")
        .with_timeout(5.0)
        .with_api_key_auth(settings.ynab_api_key)
    )
    return YnabAPIBasedRepository(
        http_client=http_client,
        budget_id=settings.ynab_budget_id,
        account_id=settings.ynab_account_id,
    )



from dataclasses import dataclass
from datetime import datetime

from dataclass_csv import accept_whitespaces, dateformat


@accept_whitespaces
@dataclass
@dateformat("%d.%m.%Y")
class SwedbankTransaction:
    client_account: str
    date: datetime
    memo: str
    amount: str
    debit_credit: str
    payee: str = ""



from dataclasses import replace
from typing import List

from ynab_import.common.models import YnabTransaction
from ynab_import.common.transformers import Transformer
from ynab_import.revolut.models import RevolutTransaction


class RevolutTransformer(Transformer):
    """Data transformation From Revolut To Ynab"""

    SKIP_KEYWORDS = ["Metal Cashback"]

    def prepare_data(self, transaction: RevolutTransaction) -> RevolutTransaction:
        paid_out_eur = (
            self.ynab_amount(transaction.paid_out_eur)
            if transaction.paid_out_eur
            else ""
        )
        paid_in_eur = (
            self.ynab_amount(transaction.paid_in_eur) if transaction.paid_in_eur else ""
        )
        description = transaction.description.strip()
        notes = transaction.notes.strip()
        return replace(
            transaction,
            description=description,
            notes=notes,
            paid_out_eur=paid_out_eur,
            paid_in_eur=paid_in_eur,
        )

    def transform(
        self, transactions: List[RevolutTransaction]
    ) -> List[YnabTransaction]:
        res = []
        for tr in transactions:
            if tr.description.strip() in self.SKIP_KEYWORDS:
                continue
            tr = self.prepare_data(tr)
            amount = tr.paid_in_eur or -1 * tr.paid_out_eur
            import_id = self.generate_import_id(
                self.account_id, amount, tr.completed_date.isoformat(),
            )
            res.append(
                YnabTransaction(
                    account_id=self.account_id,
                    date=tr.completed_date.isoformat(),
                    amount=amount,
                    payee_name=tr.description,
                    memo=tr.notes,
                    cleared=None,
                    approved=False,
                    flag_color="green",
                    import_id=import_id,
                )
            )
        return res



import abc
from collections import Counter
from typing import List

from ynab_import.common.models import YnabTransaction
from ynab_import.core.interactions import Transaction


class Transformer(abc.ABC):
    def __init__(self, account_id: str) -> None:
        self.counter: Counter = Counter()
        self.account_id = account_id

    @staticmethod
    def _to_milliunit(amount: float) -> int:
        return int(amount * 1000)

    @staticmethod
    def _to_float(amount: str) -> float:
        return float(amount.replace(",", "."))

    def ynab_amount(self, amount: str) -> int:
        if not amount.strip():
            return 0
        return self._to_milliunit(self._to_float(amount))

    def generate_import_id(self, account_id: str, amount: str, iso_date: str):
        import_id = "YNAB:{milliunit_amount}:{iso_date}:{occurrence}".format(
            milliunit_amount=amount,
            iso_date=iso_date,
            occurrence=self.counter[(account_id, amount, iso_date)],
        )
        self.counter[(account_id, amount, iso_date)] += 1
        return import_id

    @abc.abstractmethod
    def prepare_data(self, transaction: Transaction) -> Transaction:
        raise NotImplementedError

    @abc.abstractmethod
    def transform(self, transactions: List[Transaction]) -> List[YnabTransaction]:
        raise NotImplementedError



from typing import List

from dataclass_csv import DataclassReader

from ynab_import.common.providers import CSVReader
from ynab_import.core.interactions import Transaction
from ynab_import.revolut.models import RevolutTransaction


class RevolutCSVReader(CSVReader):
    COMPLETED_DATE = "Completed Date "
    DESCRIPTION = " Description "
    PAID_OUT_EUR = " Paid Out (EUR) "
    PAID_IN_EUR = " Paid In (EUR) "
    EXCHANGE_OUT = " Exchange Out"
    EXCHANGE_IN = " Exchange In"
    BALANCE_EUR = " Balance (EUR)"
    CATEGORY = " Category"
    NOTES = " Notes"

    SKIP_KEYWORDS = ["Metal Cashback"]

    def read_transactions(self, input_file: str) -> List[Transaction]:
        with open(input_file, "r", encoding="utf8") as f:
            reader = DataclassReader(
                f, RevolutTransaction, delimiter=",", quotechar='"'
            )
            reader.map(self.COMPLETED_DATE).to("completed_date")
            reader.map(self.DESCRIPTION).to("description")
            reader.map(self.PAID_OUT_EUR).to("paid_out_eur")
            reader.map(self.PAID_IN_EUR).to("paid_in_eur")
            reader.map(self.EXCHANGE_OUT).to("exchange_out")
            reader.map(self.EXCHANGE_IN).to("exchange_in")
            reader.map(self.BALANCE_EUR).to("balance_eur")
            reader.map(self.CATEGORY).to("category")
            reader.map(self.NOTES).to("notes")
            transactions = list(reader)

        return [
            self._make(tr) for tr in transactions if tr.memo not in self.SKIP_KEYWORDS
        ]

    def _make(self, transaction: RevolutTransaction) -> Transaction:
        amount = self.to_amount(transaction.amount)
        if transaction.debit_credit == "D":
            amount = amount * -1

        return Transaction(
            date=transaction.date,
            amount=amount,
            payee_name=transaction.payee,
            description=transaction.notes,
        )



from abc import ABC, abstractmethod


class CSVReader(ABC):  # pragma: no cover
    @abstractmethod
    def read_transactions(self):
        pass

    @staticmethod
    def _to_milliunit(amount: float) -> int:
        return int(amount * 1000)

    @staticmethod
    def _to_float(amount: str) -> float:
        return float(amount.replace(",", "."))

    def to_amount(self, amount: str) -> int:
        if not amount.strip():
            return 0
        return self._to_milliunit(self._to_float(amount))



from dataclasses import dataclass
from datetime import datetime

from dataclass_csv import accept_whitespaces, dateformat


@accept_whitespaces
@dataclass
@dateformat("%b %d, %Y ")
class RevolutTransaction:
    completed_date: datetime
    description: str
    paid_out_eur: str
    paid_in_eur: str
    exchange_out: str
    exchange_in: str
    balance_eur: str
    category: str
    notes: str = ""
